// Netcalc - version 1.5
// 11 December, 2007
// Copyright 2000-2007 Harry Whitfield
// mailto:g6auc@arrl.net

// -- Functions to perform electrical network calculations ---------------


// equivalent impedance of a in parallel with b

function par(a, b)
{
	if ((a.modsqr() === 0) || (b.modsqr() === 0)) { return Complex.zero; }
  	var z =  Complex.cadd(a,b);
	if (z.modsqr() === 0) { return Complex.INFINITY; }
	return Complex.cdiv(Complex.cmul(a,b),z);
}


// impedance a expressed in parallel form {R,X} such that a = {R,0} || {0,X} as string

function parform(a)
{
  	var ms = a.x*a.x + a.y*a.y;
  	if (ms === 0) { return "0,0"; }
  	return (ms/a.x).toFixed(precision) + "," + (ms/a.y).toFixed(precision);
}


// impedance a expressed in parallel form "R || X" such that a = {R,0} || {0,X} as string

function pform(a)
{
  	var ms = a.x*a.x + a.y*a.y;
  	if (ms === 0) { return "0 || 0"; }
  	return (ms/a.x).toFixed(precision) + " || " + (ms/a.y).toFixed(precision);
}


// -- Functions to perform transmission line calculations ----------------


// Reflection Coefficient rho = ((Z1/Z0) - 1) / ((Z1/Z0) + 1)

function rho(Z1,Z0)
{
  	var z = Complex.cdiv(Z1,Z0);
  	return Complex.cdiv(Complex.csub(z, Complex.one), Complex.cadd(z, Complex.one));
}


// Voltage Standing Wave Ratio VSWR = (1 + |rho|) / (1 - |rho|)

function vswr(rho)
{
  	var r = rho.modulus();
  	return (1+r)/(1-r);
}


// Return Loss = -20*log(|rho|) dB = -10*log(|rho|^2) dB

function returnLoss(rho)
{
  	return -10*log10(rho.modsqr());
}


// log to the base 10 in terms of log to the base e

function log10(x)
{
  	return Math.LOG10E * Math.log(x);
}


// input impedance of a lossless transmission line of characteristic impedance Z0
// and length x wavelengths which is terminated in a load impedance Z1

function zin(Z1, Z0, x)
{
  	var C = new Complex(Math.cos(2*Math.PI*x), 0);
  	var S = new Complex(0, Math.sin(2*Math.PI*x));
  	var Z = Complex.cdiv(Z1, Z0);
  	return Complex.cmul(Z0, Complex.cdiv(Complex.cadd(Complex.cmul(Z, C), S), Complex.cadd(C, Complex.cmul(S, Z))));
}

// input impedance of a lossy transmission line of characteristic impedance Z0
// and attenuation factor alpha (nepers/metre)
// and length x wavelengths which is terminated in a load impedance Z1

function zin2(Z1, Z0, x, alpha)
{
  	var c = 299792458; 		// speed of light in m/s
  	var v = velocity();
  	var lamda = (c*v)/frequency();
  	var length = x*lamda;
  	var arg = new Complex(alpha*length, 2*Math.PI*x);
  	var C = Complex.ccosh(arg);
  	var S = Complex.csinh(arg);
 	var Z = Complex.cdiv(Z1, Z0);
  	return Complex.cmul(Z0, Complex.cdiv(Complex.cadd(Complex.cmul(Z, C), S), Complex.cadd(C, Complex.cmul(S, Z))));
}


// length of transmission line of characteristic impedance Z0 having
// the same reactance at frequency f as the impedance Z1.

function equivLength(Z1, Z0, f, v)
{
  	var Z = Complex.cdiv(Z1, Z0);
  	var x = Math.atan(Z.y);
  	if (x < 0) { x += Math.PI; }
  	return x*v/(2*Math.PI*f);
}


// -- Functions to compute scaled values from pulldown menus -------------


function frequency()
{
  	return parseFloat(freq.data) * Math.pow(10, FMultiplier);
}


function velocity()
{
  	return parseFloat(velfac.data);
}

function resistance()
{
  	return parseFloat(resist.data) * Math.pow(10, RMultiplier);
}


function inductance()
{
  	return parseFloat(induct.data) * Math.pow(10, LMultiplier);
}


function reactance()
{
  	return parseFloat(react.data) * Math.pow(10, XMultiplier);
}


function capacitance()
{
  	return parseFloat(capacit.data) * Math.pow(10, CMultiplier);
}


function prefix(v)
{
  	var x = Math.abs(v);
  	if (x >=1000000) {return "M"; }
  	if (x >=1000) {return "k"; }
  	if (x >= 1  ) {return ""; }
  	if (x >= 0.001) {return "m"; }
  	if (x >= 0.000001) {return "µ"; }
  	if (x >= 0.000000001) {return "n"; }
  	return "p";
}


function multiplier(v)
{
  	var x = Math.abs(v);
  	if (x >= 1000000) {return 0.000001; }
  	if (x >= 1000) {return 0.001; }
  	if (x >= 1  ) {return 1.0; }
  	if (x >= 0.001) {return 1000; }
  	if (x >= 0.000001) {return 1000000; }
  	if (x >= 0.000000001) {return 1000000000; }
  	return 1000000000000;
}

// -- Functions used for button actions ----------------------------------

function toComplex(s)
{
	var item = s.split(",");
	return new Complex(Number(item[0]), Number(item[1]));
}

function update()						// update display - used after (almost) every action
{
 	Preg.data = pform(toComplex(theXreg.data));
  	Nreg.data = (Complex.cdiv(toComplex(theXreg.data),toComplex(Z0reg.data))).toString();
  
  	var rh = rho(toComplex(theXreg.data),toComplex(Z0reg.data));
  	Rreg.data = rh.modulus().toFixed(precision);

  	var theta = rh.arg();
  	if (isNaN(theta))
  	{
  		Rreg.data += " @ UNDEFINED" + degreeSign;
  	}
  	else
  	{
  		Rreg.data += " @ " + (180*theta/Math.PI).toFixed(precision) + degreeSign;
  	}

  	theZreg.data = "VSWR=" + vswr(rh).toFixed(precision) + " RL=" + returnLoss(rh).toFixed(precision) + "dB";
}

function showMod(s)						// display s in polar form in theZreg
{
	var X = toComplex(s);
  	theZreg.data = X.modulus().toFixed(precision);

  	var theta = X.arg();
  	if (isNaN(theta))
  	{
  		theZreg.data += " @ UNDEFINED" + degreeSign;
  	}
  	else
  	{
  		theZreg.data += " @ " + (180*theta/Math.PI).toFixed(precision) + degreeSign;
  	}
}

function showVSWR()
{
  	var rh = rho(toComplex(theXreg.data),toComplex(Z0reg.data));
  	theZreg.data = "VSWR=" + vswr(rh).toFixed(precision) + " RL=" + returnLoss(rh).toFixed(precision) + "dB";
}

function btnC()
{
  theXreg.data = theYreg.data;
  theYreg.data = "0,0";
  update();
}

function btnJ()
{
  theXreg.data = toComplex(theXreg.data).conj().toString();
  update();
}

function btnE()
{
  var XX = theXreg.data;
  theXreg.data = theYreg.data;
  theYreg.data = XX;
  update();
}

function btnQ()
{
	var X = toComplex(theXreg.data);
	theZreg.data = "Q=" + Math.abs(X.y / X.x).toFixed(precision);
}


function btnParallel()
{
  theXreg.data = par(toComplex(theXreg.data), toComplex(theYreg.data)).toString();
  theYreg.data = "0,0";
  update();
}

function btnSeries()
{
  theXreg.data = Complex.cadd(toComplex(theXreg.data), toComplex(theYreg.data)).toString();
  theYreg.data = "0,0";
  update();
}

function btnXL()
{
	theYreg.data = theXreg.data;
  	theXreg.data = (new Complex(resistance(), 2*Math.PI*frequency()*inductance())).toString();
  	update();
}

function btnXX()
{
	theYreg.data = theXreg.data;
  	theXreg.data = (new Complex(resistance(), reactance())).toString();
  	update();
}

function btnXC()
{
	theYreg.data = theXreg.data;
  	theXreg.data = (new Complex(resistance(), -1/(2*Math.PI*frequency()*capacitance()))).toString();
  	update();
}

function btnXLP()
{
	theYreg.data = theXreg.data;
  	var RR = new Complex(resistance(), 0);
  	var XX = new Complex(0, 2*Math.PI*frequency()*inductance());
  	theXreg.data = par(RR,XX).toString();
  	update();
}

function btnXXP()
{
	theYreg.data = theXreg.data;
  	var RR = new Complex(resistance(), 0);
  	var XX = new Complex(0, reactance());
  	theXreg.data = par(RR,XX).toString();
  	update();
}

function btnXCP()
{
	theYreg.data = theXreg.data;
  	var RR = new Complex(resistance(), 0);
  	var XX = new Complex(0, -1/(2*Math.PI*frequency()*capacitance()));
  	theXreg.data = par(RR,XX).toString();
  	update();
}

function btnmul()
{
	var factor = parseFloat(Factor.data);
	if (isNaN(factor))
	{
		beep();
	}
	else
	{
		theXreg.data = Complex.smul(factor, toComplex(theXreg.data)).toString();
  		update();
	}
}

function btndiv()
{
	var factor = parseFloat(Factor.data);
	if (isNaN(factor))
	{
		beep();
	}
	else
	{
		theXreg.data = Complex.sdiv(factor, toComplex(theXreg.data)).toString();
  		update();
	}
}

function btnL()
{
	var c = 299792458;		// speed of light in m/s
	theZreg.data = "Length=" + (1000*equivLength(toComplex(theXreg.data), toComplex(Z0reg.data), frequency(), c*velocity())).toFixed(1) + "mm";
}

function menuRn(i)
{
	theYreg.data = theXreg.data;
	theXreg.data = Mreg[i];
    update();
}

function menuQn(i)
{
	theZreg.data = Mreg[i];
}

function menuMn(i)
{
	Mreg[i] = theXreg.data;
}

function btnHelp()
{
	var path = "Resources/Help/NetCalc.help";
	
	if (system.platform != "macintosh")
	{
		path = "Resources/Help/NetCalc.html";
	}
	filesystem.open(path);
}

function btnX()
{
	var X = toComplex(theXreg.data);
	var it = X.y;
    if (it >= 0)
    {
    	it = it/(2*Math.PI*frequency());
    	theZreg.data = "Lx=" + (it*multiplier(it)).toFixed(precision) + prefix(it) + "H";
    }
  	else
  	{
  		it = -1/(2*Math.PI*frequency()*it);
   		theZreg.data = "Cx=" + (it*multiplier(it)).toFixed(precision) + prefix(it) + "F";
 	}
  
  	it = toComplex(parform(X)).y;
	
	if (isFinite(it))
	{
    	if (it >= 0)
    	{
      		it = it/(2*Math.PI*frequency());
      		theZreg.data += "  Lp=" + (it*multiplier(it)).toFixed(precision) + prefix(it) + "H";
      	}
    	else
    	{
      		it = -1/(2*Math.PI*frequency()*it);
      		theZreg.data += "  Cp=" + (it*multiplier(it)).toFixed(precision) + prefix(it) + "F";
      	}
	}
	else
	{
    	theZreg.data += "  Xp=Infinite";
    }
}

function btnT(optionKey)
{
  	var it;
  	var Z0 = toComplex(Z0reg.data);
  	var Z1 = toComplex(theXreg.data);
	var x = parseFloat(Tlength.data);
	var v = 0;

	if (isNaN(x)) { beep(); }
	else
	{
		if (x >= 1) 			//   x in mm
    	{
    		c = 299792458; 		// speed of light in m/s
			v = velocity();
    		x = 0.001*x*frequency()/(c*v);
  		}
  
  		if (optionKey)			// lossy transmission line
  		{
			it =  parseFloat(Factor.data);
    		if (isNaN(it)) { beep(); }
    		else
    		{
	    		theXreg.data = zin2(Z1, Z0, x, it).toString();
	    		update();
	   		}
	   	}
  		else					// lossless transmission line
  		{
    		theXreg.data = zin(Z1, Z0, x).toString();
    		update();
 		}
  	}
}

function btnLS(optionKey)
{
	var Z0 = toComplex(Z0reg.data);
	var X =  toComplex(theXreg.data);
	var omega =  2*Math.PI*frequency();

  	computeLS(optionKey, omega, Z0.x, X.x, X.y);
  
	Results.opacity = 255;
	Results.bgOpacity = 255;
	Results.zOrder = 200;
}

function say(optionKey, x, omega)
{
  	var it = 0;
  	
  	if (optionKey)
  	{
    	it = x;
    	Results.data += "reactance is " + (it*multiplier(it)).toPrecision(6) + prefix(it) + ohmSign + "\n";
  	}
	else
	{
    	if (x < 0)
    	{
      		it = -1/(x * omega);
      		Results.data += "capacitance is " + (it*multiplier(it)).toPrecision(6) + prefix(it) + "F\n";
    	}
		else
		{
      		it = x/omega;
      		Results.data += "inductance  is " + (it*multiplier(it)).toPrecision(6) + prefix(it) + "H\n";
    	}
  	}
}

function computeLS(optionKey, omega, R0, R, X)
{
  	Results.data = "";
  	
  	var A = R*R + X*X;
  	var G = R/A;
  	var B = -X/A;
  
  	Results.data += "Shunt arm parallel to load\n";
  	
  	var P = 1/(G*R0);
  	
  	var X1 = 0;
  	var B2 = 0;
  	var X2 = 0;
  	
  	if (P < 1) 
  	{
    	Results.data += "No solution\n";
  	}
	else
	{
    	A = Math.sqrt(P - 1);
    	X1 = R0*A;
    	B2 = -A/(P*R0);
    	
    	Results.data += "First solution\n  Series arm ";
       	say(optionKey, -X1, omega);
       	Results.data += "  Shunt  arm ";
       	say(optionKey, -1/(B2 - B), omega);
    	Results.data += "Second solution\n  Series arm ";
       	say(optionKey, X1, omega);
       	Results.data += "  Shunt  arm ";
       	say(optionKey, -1/(-B2 - B), omega);
  	}
   	Results.data += "\n";
  	
  	Results.data += "Shunt arm parallel to generator\n";
  	
  	P = R0/R;
  	
  	if (P < 1)
  	{
   	 	Results.data += "No solution\n";
  	}
	else
	{
    	A = Math.sqrt(P - 1);
    	X1 = R*A;
    	X2 = P*R/A;
    	Results.data += "First solution\n  Series arm ";
    	say(optionKey, -X1 - X, omega);
    	Results.data += "  Shunt  arm ";
    	say(optionKey, X2, omega);
   		Results.data += "Second solution\n  Series arm ";
    	say(optionKey, X1 - X, omega);
    	Results.data += "  Shunt  arm ";
    	say(optionKey, -X2, omega);
  	}
}

function btnClose()
{
	closeWidget();
}

function btnRight()
{
	preferences.overlapPref.value = "To right of calculator";
	savePreferences();
	checkChart();
}

function btnLeft()
{
	preferences.overlapPref.value = "On top of calculator";
	savePreferences();
	checkChart();
}
